"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MakeToJsonCommand = exports.MakeToJsCommand = void 0;
const vscode = require("vscode");
const path = require("path");
const fs_1 = require("../utils/fs");
function handlePath(mystring) {
    const platform = process.platform;
    if (platform === 'win32') {
        let p = mystring.replace(/^\/([^\/]?)\//, function (_match, drive) {
            return (drive || 'c').toUpperCase() + ':\\';
        }).replace(/\//g, '\\');
        if (p.startsWith("\\")) {
            return p.substring(1);
        }
        else {
            return p;
        }
    }
    else {
        return mystring;
    }
}
function getLocalCmdAtPath(folderPath, cmd, cmdWin32) {
    return __awaiter(this, void 0, void 0, function* () {
        const platform = process.platform;
        const bin = path.join(folderPath, 'node_modules', '.bin');
        if (platform === 'win32' && (yield fs_1.exists(vscode.Uri.file(path.join(bin, cmdWin32))))) {
            return handlePath(path.join(bin, cmdWin32));
        }
        else if ((platform === 'linux' || platform === 'darwin' || platform === 'android') && (yield fs_1.exists(vscode.Uri.file(path.join(bin, cmd))))) {
            return path.join(bin, cmd);
        }
        return undefined;
    });
}
function getCmdPath(file, cmd, cmdWin32) {
    return __awaiter(this, void 0, void 0, function* () {
        let lastDir = file;
        do {
            let dir = path.dirname(lastDir);
            if (dir === lastDir) {
                return undefined;
            }
            lastDir = dir;
            let p = yield getLocalCmdAtPath(dir, cmd, cmdWin32);
            if (p) {
                return { dir, jtu: p };
            }
        } while (true);
    });
}
function runMakejs(jtu, cwd, file) {
    let cmd;
    if (file.endsWith(".程")) {
        cmd = `${jtu} -t ES3 --jsx react --skipLibCheck true --esModuleInterop true --reactNamespace 回应 ${file}`;
    }
    else {
        cmd = `${jtu} -t ES3 --skipLibCheck true --esModuleInterop true ${file}`;
    }
    let task = new vscode.Task({ type: 'shell' }, vscode.TaskScope.Workspace, '生成脚本(js)', 'JtScript', new vscode.ShellExecution(cmd, { cwd: cwd }));
    vscode.tasks.executeTask(task);
    // child_process.exec(cmd, { cwd }, (err, stdout, stderr) => {
    // 	console.log('stdout: ' + stdout);
    // 	console.log('stderr: ' + stderr);
    // 	if (err) {
    // 		console.log('error: ' + err);
    // 	}
    // });
}
class MakeToJsCommand {
    constructor() {
        this.id = 'jtscript.makeToJs';
    }
    execute(uri) {
        if (uri) {
            getCmdPath(uri.path, 'jtu', 'jtu.cmd').then(p => {
                console.log(p);
                if (p) {
                    runMakejs(p.jtu, p.dir, handlePath(uri.path));
                }
            }).catch(e => {
                console.log(e);
            });
        }
    }
}
exports.MakeToJsCommand = MakeToJsCommand;
function runMakeJson(jtu, file) {
    let baseName = path.basename(file);
    let cmd = `${jtu} ${baseName}`;
    let task = new vscode.Task({ type: 'shell' }, vscode.TaskScope.Workspace, '生成输出文件', 'JtScript', new vscode.ShellExecution(cmd, { cwd: path.dirname(file) }));
    vscode.tasks.executeTask(task);
    // var extension = path.extname(file);
    // let baseName = path.basename(file, extension);
    // let cmd = `${jtu} ${baseName}`;
    // child_process.exec(cmd, { cwd: path.dirname(file) }, (err, stdout, stderr) => {
    // 	console.log('stdout: ' + stdout);
    // 	console.log('stderr: ' + stderr);
    // 	if (err) {
    // 		console.log('error: ' + err);
    // 	}
    // });
}
class MakeToJsonCommand {
    constructor() {
        this.id = 'jtscript.makeToOutputFile';
    }
    execute(uri) {
        if (uri) {
            getCmdPath(uri.path, 'jtpeizhi', 'jtpeizhi.cmd').then(p => {
                console.log(p);
                if (p) {
                    runMakeJson(p.jtu, handlePath(uri.path));
                }
            }).catch(e => {
                console.log(e);
            });
        }
    }
}
exports.MakeToJsonCommand = MakeToJsonCommand;
//# sourceMappingURL=makeToJs.js.map