"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TsConfigProvider = void 0;
const vscode = require("vscode");
class TsConfigProvider {
    getConfigsForWorkspace(token) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!vscode.workspace.workspaceFolders) {
                return [];
            }
            const configs = new Map();
            for (const config of yield this.findConfigFiles(token)) {
                const root = vscode.workspace.getWorkspaceFolder(config);
                if (root) {
                    configs.set(config.fsPath, {
                        uri: config,
                        fsPath: config.fsPath,
                        posixPath: config.path,
                        workspaceFolder: root
                    });
                }
            }
            return configs.values();
        });
    }
    findConfigFiles(token) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield vscode.workspace.findFiles('**/jtconfig*.json', '**/{node_modules,.*}/**', undefined, token);
        });
    }
}
exports.TsConfigProvider = TsConfigProvider;
//# sourceMappingURL=tsconfigProvider.js.map